require 'bigdecimal'

module Rledger
  # Amount stores an amount, as an Hash currency => value.
  #
  # The peculiar storage simplifies operations on multiple currencies 
  # (adding different currencies is equivalent to merging hashes)
  #
  class Amount
    def initialize
      @amount = Hash.new
    end

    def parse(s)
      currency = "([a-zA-Z$]+|)"
      amount = "(-?[0-9]+\\.?[0-9]+|)"
      ob = "[\t ]*"
      
      match = Regexp.new(currency + ob + amount).match(s)

      if match
        currency = match[1]
        amount   = match[2] == "" ?  BigDecimal.new('0.00') : BigDecimal.new(match[2])
        @amount[currency] = amount
        true
      else
        false
      end
    end

    def to_s
      @amount.keys.collect { |key| key + " " + "%.2f" % @amount[key] }.join(", ")
    end

    def add!(other_amount)
      @amount.merge!(other_amount.hash) { |key, oldval, newval| @amount[key] = oldval + newval } 
      self
    end

    def multiply!(factor)
      @amount.keys.map { |x| @amount[x] = factor * @amount[x] }
      self
    end

    def single_currency?
      @amount.keys.size == 1
    end

    def currencies 
      @amount.keys
    end

    def amount_of currency
      @amount[currency]
    end

    # good only if single amount
    def amount
      @amount[@amount.keys[0]]
    end

    # good only if single currency
    def currency
      @amount.keys[0]
    end

    def hash
      @amount
    end
  end
end
